@@ -55,7 +55,7 @@ static long crbs_ast_heuristic(usrch_a_star_t *ctx, void *node)
 	dist = crbs_dist_heur(crbs->target->pt->x, crbs->target->pt->y, tx, ty);
 
  /* prefer concave over convex assuming the puller will get it shorter */
-	if (((from->type & 0x0F) == ADDR_ARC_VCONCAVE) || ((from->type & 0x0F) == ADDR_ARC_CONCAVE))
+	if ((from->type & 0x0F) == ADDR_ARC_VCONCAVE)
 		dist = dist * 2 / 3;
 
 	return dist;
@@ -73,7 +73,6 @@ static long crbs_ast_cost(usrch_a_star_t *ctx, void *from_, void *to_)
 
 	switch(from->type & 0x0F) {
 		case ADDR_ARC_VCONCAVE:
-		case ADDR_ARC_CONCAVE:
 			tune = 2.0/3.0; /* prefer concave over convex assuming the puller will get it shorter */
 		case ADDR_ARC_CONVEX:
 			ex = from->pt->x + cos(from->new_sa + from->new_da) * from->new_r;
@@ -91,7 +90,6 @@ static long crbs_ast_cost(usrch_a_star_t *ctx, void *from_, void *to_)
 
 	switch(to->type & 0x0F) {
 		case ADDR_ARC_VCONCAVE:
-		case ADDR_ARC_CONCAVE:
 			tune = tune * (2.0/3.0);
 		case ADDR_ARC_CONVEX:
 			to_dir = to->new_adir;
@@ -118,7 +116,6 @@ static long crbs_ast_cost(usrch_a_star_t *ctx, void *from_, void *to_)
 	ctn = crbs->routing_tn->user_data;
 	if (cto->net == ctn->net) {
 		switch(to->type & 0x0F) {
-			case ADDR_ARC_CONCAVE:
 			case ADDR_ARC_CONVEX:
 				cost *= 3;
 				break;
@@ -209,6 +206,8 @@ static int crbs_is_spiral(usrch_a_star_t *ctx, grbs_detached_addr_t *froma, grbs
 		if (da->pt == toa->obj.arc->parent_pt)
 			return 1;
 	}
+
+	return 0;
 }
 
 static int hop_cnt = 0;
@@ -259,7 +258,7 @@ static grbs_detached_addr_t *crbs_next_hop(usrch_a_star_t *ctx, grbs_detached_ad
 			crbs_grbs_draw(crbs, tmp);
 		}
 
-		if (((froma->type & 0x0f) == ADDR_ARC_CONVEX) || ((froma->type & 0x0f) == ADDR_ARC_CONCAVE)) {
+		if ((froma->type & 0x0f) == ADDR_ARC_CONVEX) {
 			if (fabs(froma->obj.arc->new_da) > M_PI) {
 /*				printf("   INVALID arc (too long): %f at P%ld\n", froma->obj.arc->new_da, froma->obj.arc->parent_pt == NULL ? -1 : froma->obj.arc->parent_pt->uid);*/
 				grbs_path_cleanup_addr(&crbs->grbs, toa);
@@ -268,15 +267,6 @@ static grbs_detached_addr_t *crbs_next_hop(usrch_a_star_t *ctx, grbs_detached_ad
 			}
 		}
 
-		if ((froma->type & 0x0f) == ADDR_ARC_CONCAVE) {
-			if (fabs(froma->obj.arc->new_da) < 0.01) {
-/*				printf("   INVALID arc (too short): %f at P%ld\n", froma->obj.arc->new_da, froma->obj.arc->parent_pt == NULL ? -1 : froma->obj.arc->parent_pt->uid);*/
-				grbs_path_cleanup_addr(&crbs->grbs, toa);
-				grbs_path_cleanup_addr(&crbs->grbs, froma);
-				return NULL;
-			}
-		}
-
 		if ((adir == GRBS_ADIR_CONVEX_CCW) || (adir == GRBS_ADIR_CONVEX_CW)) {
 			if (crbs_is_spiral(ctx, curr, toa)) {
 				printf("   INVALID spiral: P%ld\n", froma->obj.arc->parent_pt == NULL ? -1 : froma->obj.arc->parent_pt->uid);
@@ -330,7 +320,7 @@ static void *crbs_ast_neighbor(usrch_a_star_t *ctx, void *curr_, void *nctx)
 	for(;;) {
 		crbs_point_t *pt;
 
-		if (a->grbs_idx >= 5) {
+		if (a->grbs_idx >= 3) {
 			a->grbs_idx = 0;
 			a->pt_idx++;
 		}
@@ -342,7 +332,6 @@ static void *crbs_ast_neighbor(usrch_a_star_t *ctx, void *curr_, void *nctx)
 
 		switch(curr->type & 0x0F) {
 			case ADDR_ARC_CONVEX:   from_type = (curr->new_adir > 0) ? "convex cw" : "convex ccw"; break;
-			case ADDR_ARC_CONCAVE:  from_type = (curr->new_adir > 0) ? "concave cw" : "concave ccw"; break;
 			case ADDR_ARC_VCONCAVE: from_type = "vconcave"; break;
 			case ADDR_POINT:        from_type = "incident"; break;
 			default:                from_type = "UNKNOWN"; break;
@@ -364,7 +353,7 @@ static void *crbs_ast_neighbor(usrch_a_star_t *ctx, void *curr_, void *nctx)
 		}
 
 
-		while(a->grbs_idx < 5) {
+		while(a->grbs_idx < 3) {
 			if (crbs_trace_ast) {
 				if ((curr->type & 0x0F) == ADDR_ARC_VCONCAVE)
 					printf(" try from %s P%ld (real %.2f %.2f) to P%ld", from_type, curr->pt->uid, curr[1].pt->x, curr[1].pt->y, pt->gpt->uid);
@@ -372,24 +361,16 @@ static void *crbs_ast_neighbor(usrch_a_star_t *ctx, void *curr_, void *nctx)
 					printf(" try from %s P%ld to P%ld", from_type, curr->pt->uid, pt->gpt->uid);
 			}
 			if (crbs->disable_vconcave) {
-				if (a->grbs_idx < 3)
-					a->grbs_idx = 3;
+				if (a->grbs_idx < 1)
+					a->grbs_idx = 1;
 			}
 
-			if (crbs->disable_concave) {
-				if (a->grbs_idx < 2)
-					a->grbs_idx = 2;
-			}
-
-
 			/* prefer concave over convex; order matters when reaching target: first
 			   valid solution will stop the search */
 			switch(a->grbs_idx) {
-				case 0: if (crbs_trace_ast) printf(" concave ccw"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_CONCAVE_CCW); break;
-				case 1: if (crbs_trace_ast) printf(" concave cw"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_CONCAVE_CW);  break;
-				case 2: if (crbs_trace_ast) printf(" vconcave"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_VCONCAVE); break;
-				case 3: if (crbs_trace_ast) printf(" convex ccw"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_CONVEX_CCW); break;
-				case 4: if (crbs_trace_ast) printf(" convex cw"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_CONVEX_CW); break;
+				case 0: if (crbs_trace_ast) printf(" vconcave"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_VCONCAVE); break;
+				case 1: if (crbs_trace_ast) printf(" convex ccw"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_CONVEX_CCW); break;
+				case 2: if (crbs_trace_ast) printf(" convex cw"); next = crbs_next_hop(ctx, curr, pt, GRBS_ADIR_CONVEX_CW); break;
 			}
 
 			a->grbs_idx++;
@@ -576,8 +557,6 @@ static int rt_topo_crbs_route_net(crbs_t *crbs, crbs_2net_t *ctn)
 			case GRBS_ADIR_INC:         printf(" to P%ld", da->pt->uid); break;
 			case GRBS_ADIR_CONVEX_CCW:  printf(" ccw P%ld", da->pt->uid); break;
 			case GRBS_ADIR_CONVEX_CW:   printf(" cw P%ld", da->pt->uid); break;
-			case GRBS_ADIR_CONCAVE_CCW: printf(" concave ccw P%ld", da->pt->uid); break;
-			case GRBS_ADIR_CONCAVE_CW:  printf(" concave cw P%ld", da->pt->uid); break;
 			case GRBS_ADIR_VCONCAVE:    printf(" vconcave P%ld", da->pt->uid); break;
 		}
 		na = grbs_path_next(&crbs->grbs, crbs->routing_tn, pa, da->pt, da->user_long);
@@ -740,8 +719,6 @@ static int rt_topo_crbs_layer(rtrnd_t *ctx, rtrnd_layer_t *ly, gdl_list_t *tnl)
 
 	memset(&crbs, 0, sizeof(crbs_t));
 	crbs.disable_vconcave = 0;
-	crbs.disable_concave = 1;
-	crbs.enable_mid_virt = rt_topo_cfg.mid_virt;
 	crbs_init(&crbs, ctx);
 
 	rt_topo_crbs_cdt_init(ctx, &crbs);
