@@ -18,7 +18,7 @@ typedef struct {
 typedef struct {
 	/* configured */
 	double wire_thick, wire_clr, via_dia, via_clr, beta;
-	int octilin, algo, mid_virt;
+	int octilin, algo;
 	double max_hop_mult, junction_penalty, via_via_space;
 
 	/* calculated */
