@@ -367,7 +367,7 @@ static trbs_cross_t *trbs_vis_jump_edge(trbs_t *trbs, triangle_t *t, trbs_cross_
 		/* this point may be our target point */
 		if (corner == trbs->target) {
 			if (trbs_vis_trace) tprintf(" -> target corner! (P%ld)\n", ((trbs_point_t *)corner->data)->uid);
-			return trbs->target;
+			return (trbs_cross_t *)trbs->target; /* caller checks for this ptr explicitly */
 		}
 
 		if (dstdir > 0) {
@@ -478,7 +478,7 @@ static trbs_cross_t *trbs_is_visible_from_edge(trbs_t *trbs, triangle_t *t, edge
 	/* go around in CCW to find our target edge */
 	for(timeout = 0; timeout < 3; timeout++) {
 		cr = trbs_vis_jump_edge(trbs, t, cr);
-		if (cr == trbs->target)
+		if (cr == (trbs_cross_t *)trbs->target)
 			return cr;
 
 #if 0
@@ -575,7 +575,7 @@ void trbs_next_edge_from_edge(trbs_t *trbs, vtp0_t *dst, triangle_t *t, trbs_cro
 		/* determine the next crossing */
 		cr = trbs_vis_jump_edge(trbs, t, cr);
 
-		if (cr == trbs->target) {
+		if (cr == (trbs_cross_t *)trbs->target) {
 			vtp0_append(dst, cr);
 			break;
 		}
