@@ -1,3 +1,26 @@
+/*  libgrbs - geometric rubber band sketch model
+    Copyright (C) 2021  Tibor 'Igor2' Palinkas
+    (Supported by NLnet NGI0 PET Fund in 2021)
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+    Contact:
+      Project page: http://repo.hu/projects/libgrbs
+      lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+*/
+
 /*#define GRBS_ROUTE_REALIZE_TRACE*/
 
 #undef tprinf
@@ -23,10 +46,10 @@ static void update_seg_sentinel_angles(grbs_arc_t *sentinel, grbs_arc_t *narc)
 		double sentinel_ea = sentinel->sa + sentinel->da;
 
 		/* normalize start angle */
-		if (sa > 2*M_PI)
-			sa -= 2*M_PI;
+		if (sa > 2*GRBS_PI)
+			sa -= 2*GRBS_PI;
 		else if (sa < 0)
-			sa += 2*M_PI;
+			sa += 2*GRBS_PI;
 
 		/* extend the original sentinel instead of overwriting; however, this
 		   normally shouldn't happen as an unfinished net's dangling endpoint
@@ -56,7 +79,7 @@ static void auto_update_seg_sentinel_angles(grbs_arc_t *narc)
 static int grbs_angle_update_get_ang_side(grbs_arc_t *arc, int at_end, double olda[2], double a[2])
 {
 	int ang_side;
-	double ta, best_ta = 4 * M_PI, diff, real_ta;
+	double ta, best_ta = 4 * GRBS_PI, diff, real_ta;
 
 	if (at_end)
 		ta = arc->sa + arc->da;
@@ -64,7 +87,7 @@ static int grbs_angle_update_get_ang_side(grbs_arc_t *arc, int at_end, double ol
 		ta = arc->sa;
 
 	if (ta < 0)
-		ta += 2.0 * M_PI;
+		ta += 2.0 * GRBS_PI;
 
 	/* check ang_side for 0 and 1 to see if ta is closer: */
 
@@ -74,11 +97,11 @@ static int grbs_angle_update_get_ang_side(grbs_arc_t *arc, int at_end, double ol
 	real_ta = ta;
 
 	/* ... then check the other combinations of ang_side ang ta vs. ta - 2*pi */
-	diff = fabs(olda[0] - (ta - 2.0 * M_PI));
+	diff = fabs(olda[0] - (ta - 2.0 * GRBS_PI));
 	if (diff < best_ta) {
 		ang_side = 0;
 		best_ta = diff;
-		real_ta = ta - 2.0 * M_PI;
+		real_ta = ta - 2.0 * GRBS_PI;
 	}
 
 	diff = fabs(olda[1] - ta);
@@ -88,11 +111,11 @@ static int grbs_angle_update_get_ang_side(grbs_arc_t *arc, int at_end, double ol
 		real_ta = ta;
 	}
 
-	diff = fabs(olda[1] - (ta - 2.0 * M_PI));
+	diff = fabs(olda[1] - (ta - 2.0 * GRBS_PI));
 	if (diff < best_ta) {
 		ang_side = 1;
 		best_ta = diff;
-		real_ta = ta - 2.0 * M_PI;
+		real_ta = ta - 2.0 * GRBS_PI;
 	}
 
 
@@ -142,31 +165,13 @@ static int grbs_angle_update_ang1(grbs_t *grbs, grbs_arc_t *arc, int at_end, grb
 		crossbelt = 1;
 
 	nextr = next->r;
-	if (next->concave)
-		nextr = 0;
 
 	if (grbs_bicycle_angles(arc->parent_pt->x, arc->parent_pt->y, oldr, next->parent_pt->x, next->parent_pt->y, nextr, olda, crossbelt) != 0)
 		return 0;
 	if (grbs_bicycle_angles(arc->parent_pt->x, arc->parent_pt->y, arc->r, next->parent_pt->x, next->parent_pt->y, nextr, a, crossbelt) != 0)
 		return 0;
 
-	if (next->concave) {
-		double ox, oy, ta;
-		cnc_get_end_arc(arc, at_end, &ox, &oy);
-		ta = -atan2(next->parent_pt->y - oy, next->parent_pt->x - ox);
-		if (ta < 0)
-			ta += 2.0 * M_PI;
-		if (fabs(ta - next->sa) < 0.001)
-			next_ang_side = 0;
-		else {
-			assert(fabs(ta - (next->sa + next->da)) < 0.001);
-			next_ang_side = 1;
-		}
-
-		olda[2+next_ang_side] = ta;
-		next_at_end = next_ang_side;
-	}
-	else {
+	{
 		next_at_end = !at_end;
 		if (next->r != 0)
 			next_ang_side = grbs_angle_update_get_ang_side(next, next_at_end, olda+2, a+2);
@@ -190,9 +195,6 @@ static void grbs_angle_update(grbs_t *grbs, grbs_arc_t *arc, double oldr)
 	grbs_arc_t *next, *prev;
 	int changed = 0;
 
-	if (arc->concave)
-		return; /* concave arcs keep their angles on move */ /* concave-target-angle */
-
 	next = arc->link_2net.next;
 	prev = arc->link_2net.prev;
 
@@ -205,7 +207,7 @@ static void grbs_angle_update(grbs_t *grbs, grbs_arc_t *arc, double oldr)
 }
 
 /* Calculate incident angles (e.g. on start/end, after a route has been realized) */
-static void grbs_inc_ang_update(grbs_t *grbs, grbs_arc_t *a)
+void grbs_inc_ang_update(grbs_t *grbs, grbs_arc_t *a)
 {
 	double cx, cy, nx, ny, ang;
 	grbs_arc_t *next;
@@ -236,11 +238,18 @@ static void grbs_inc_ang_update(grbs_t *grbs, grbs_arc_t *a)
 
 static void realize_maybe_detach(grbs_t *grbs, grbs_arc_t *arc)
 {
-	if ((arc != NULL) && (arc->r > 0) && !arc->concave) {
+	if ((arc != NULL) && (arc->r > 0)) {
 		double min_r2 = grbs_self_isect_convex_r2(grbs, arc);
 		if (arc->r * arc->r < min_r2) {
 			tprintf("   realize_maybe_detach: too small, would become concave, detach (%f;%f)!\n", arc->parent_pt->x, arc->parent_pt->y);
-			return grbs_force_detach(grbs, arc);
+			grbs_force_detach(grbs, arc);
+			return;
+		}
+		if ((arc->da < -5.5) || (arc->da > 5.5)) {
+			tprintf("   realize_maybe_detach: invalid delta angle\n");
+#warning TODO: it would be better to handle this where it happens: grbs_angle_update() on other-side (next or prev) arc
+			grbs_force_detach(grbs, arc);
+			return;
 		}
 	}
 }
@@ -316,8 +325,7 @@ static int bump_seg_radii(grbs_t *grbs, grbs_arc_t *from, double incr, double la
 				a->bbox = old_bbox;
 
 				if (cr) {
-#warning TODO: coll
-					tprintf("TODO: fail/collision\n");
+					tprintf("fail/collision\n");
 					return 1;
 				}
 				return 0;
@@ -327,8 +335,7 @@ static int bump_seg_radii(grbs_t *grbs, grbs_arc_t *from, double incr, double la
 		/* there was no existing arc above ours in the same sector; this means
 		   our new arc is on top, check it */
 		if (coll_check_arc(grbs, grbs_arc_parent_2net(a), a, 1)) {
-#warning TODO: coll
-			tprintf("TODO: fail/collision\n");
+			tprintf("fail/collision\n");
 			return 1;
 		}
 	}
@@ -336,37 +343,10 @@ static int bump_seg_radii(grbs_t *grbs, grbs_arc_t *from, double incr, double la
 	return 0;
 }
 
-/* Calculate the minimum r for the lowest concave orbit from the highest
-   convex arc and current tn; this shall be done only when tn is inserted
-   as the lowest concave orbit */
-static double bump_get_highest_convex(grbs_t *grbs, grbs_2net_t *tn, grbs_arc_t *conc_sentinel, double *highest_copper, double *highest_clearance)
-{
-	int conv_segi;
-	double highest = 0, hnewr;
-
-	for(conv_segi = 0; conv_segi < GRBS_MAX_SEG; conv_segi++) {
-		grbs_arc_t *convex_last;
-		grbs_arc_t *conv_sentinel = gdl_first(&conc_sentinel->parent_pt->arcs[0][conv_segi]);
-
-		if ((conv_sentinel == NULL) || (!grbs_arc_overlaps_arc_new(conv_sentinel, conc_sentinel))) continue;
-
-		convex_last = gdl_last(&conv_sentinel->parent_pt->arcs[0][conv_segi]);
-
-		hnewr = convex_last->r + tn->copper + convex_last->copper + GRBS_MAX(tn->clearance, convex_last->clearance);
-		if (hnewr > highest) {
-			highest = hnewr;
-			*highest_copper = convex_last->copper;
-			*highest_clearance = convex_last->clearance;
-		}
-	}
-
-	return highest;
-}
-
 /* if there is anything above the new arc, make room for it by
    pushing the above-arcs further (by exactly as much as the new arc
    is going to take) */
-static double bump_seg_radii_any(grbs_t *grbs, grbs_2net_t *tn, grbs_arc_t *arc, int *need_narc_post_update, double *narc_oldr_out, int is_concave, int dry, int *coll_out)
+static double bump_seg_radii_any(grbs_t *grbs, grbs_2net_t *tn, grbs_arc_t *arc, int *need_narc_post_update, double *narc_oldr_out, int dry, int *coll_out)
 {
 	grbs_arc_t *next;
 	double prev_copper, prev_clearance, newr, incr, prevr;
@@ -374,7 +354,7 @@ static double bump_seg_radii_any(grbs_t *grbs, grbs_2net_t *tn, grbs_arc_t *arc,
 
 	*coll_out = 0;
 
-	/* simple case: either convex or concave without convex segments below:
+	/* simple case: convex without convex segments below:
 	   previous radius:copper:clearance is the same-segment arc below or the point */
 	if (!arc->in_use) {
 		prev_copper = arc->parent_pt->copper;
@@ -387,17 +367,6 @@ static double bump_seg_radii_any(grbs_t *grbs, grbs_2net_t *tn, grbs_arc_t *arc,
 		prevr = arc->r;
 	}
 
-		/* when adding a new concave on bottom it's radius may depend on any
-		   convex region that overlaps with ours */
-	if (is_concave && (arc->link_point.prev == NULL)) {
-		double cop, clr, hnewr;
-		hnewr = bump_get_highest_convex(grbs, tn, arc, &cop, &clr);
-
-		/* new radius of our concave arc if it was only for the highest convex below */
-		if ((hnewr > 0) && (hnewr > narcr))
-			narcr = hnewr;
-	}
-
 	next = grbs_next_arc_in_use(arc);
 	if (next == NULL) {
 		*narc_oldr_out = 0;
@@ -423,83 +392,33 @@ static double bump_seg_radii_any(grbs_t *grbs, grbs_2net_t *tn, grbs_arc_t *arc,
 	return narcr;
 }
 
-/* bump any concave segment that is in overlap with our convex */
-static void bump_seg_radii_concave_above_convex(grbs_t *grbs, grbs_arc_t *convex_sentinel, int dry, int *coll_out)
-{
-	int conc_segi;
-
-	for(conc_segi = 0; conc_segi < GRBS_MAX_SEG; conc_segi++) {
-		double newr, enda;
-		grbs_arc_t *convex_last, *conc_first;
-		grbs_arc_t *conc_sentinel = gdl_first(&convex_sentinel->parent_pt->arcs[1][conc_segi]);
-
-		if ((conc_sentinel == NULL) || (!grbs_arc_overlaps_arc(convex_sentinel, conc_sentinel))) continue;
-
-		convex_last = gdl_last(&convex_sentinel->parent_pt->arcs[0][convex_sentinel->segi]);
-		conc_first = conc_sentinel->link_point.next;
-
-		if (conc_first == NULL)
-			return; /* orphaned concave */
-
-		newr = convex_last->r + convex_last->copper + conc_first->copper + GRBS_MAX(convex_last->clearance, conc_first->clearance); /* new arc added on top of the previous */
-		if (newr > conc_first->r)
-			*coll_out = bump_seg_radii(grbs, conc_first, newr - conc_first->r, -1, -1, 0, dry);
-
-		/* the new convex may need to move the endpoint of the first concave so
-		   the exit arc-line corner of the concave is not too close to the convex */
-		if (grbs_angle_in_arc(conc_sentinel->sa, conc_sentinel->da, convex_last->sa, 0)) {
-			if (try_extend_concave(grbs, conc_first, convex_last->new_sa, 0, (convex_last->da > 0) ? +1 : -1, 0), dry) {
-#warning TODO: tune concaves above, tune other side
-			}
-			else {
-				tprintf("bump_seg_radii_concave_above_convex: failed to tune concave-above-convex: start\n");
-			}
-		}
-
-		enda = convex_last->sa + convex_last->da;
-		if (enda > 2*M_PI) enda -= 2*M_PI;
-		if (enda < 0) enda += 2*M_PI;
-
-		if (grbs_angle_in_arc(conc_sentinel->sa, conc_sentinel->da, enda, 0)) {
-			if (try_extend_concave(grbs, conc_first, enda, 1, (convex_last->da > 0) ? +1 : -1, 0), dry) {
-#warning TODO: tune concaves above, tune other side
-			}
-			else {
-				tprintf("bump_seg_radii_concave_above_convex: failed to tune concave-above-convex: end\n");
-			}
-		}
-
-	}
-}
-
 static grbs_arc_t *grbs_path_realize_(grbs_t *grbs, grbs_2net_t *tn, grbs_addr_t *addr, int reverse, int dry, grbs_arc_t *ins_after)
 {
 	grbs_arc_t *arc, *narc, *a, *first;
 	static grbs_arc_t narc_dry;
 	grbs_point_t *pt;
-	int rev = 1, need_narc_post_update = 0, type = (addr->type & 0x0F), is_concave = (type == ADDR_ARC_CONCAVE);
+	int rev = 1, need_narc_post_update = 0, type = (addr->type & 0x0F);
 	int coll = 0, coll_tmp;
 	double narcr, narc_oldr;
 
 	switch(type) {
 		case ADDR_ARC_CONVEX:
-		case ADDR_ARC_CONCAVE:
 			arc = addr->obj.arc;
 			assert(arc->new_in_use);
 
 			pt = arc->parent_pt;
-			first = gdl_first(&pt->arcs[is_concave][arc->segi]);
+			first = gdl_first(&pt->arcs[arc->segi]);
 
 			/* make room for the new arc */
-			narcr = bump_seg_radii_any(grbs, tn, arc, &need_narc_post_update, &narc_oldr, is_concave, dry, &coll_tmp);
+			narcr = bump_seg_radii_any(grbs, tn, arc, &need_narc_post_update, &narc_oldr, dry, &coll_tmp);
 			coll |= coll_tmp;
 
 			/* now that we have the room: insert the new arc */
 			if (!dry) {
 				if (rev)
-					narc = grbs_arc_new(grbs, arc->parent_pt, arc->segi, narcr, arc->new_sa + arc->new_da, -arc->new_da, arc->concave);
+					narc = grbs_arc_new(grbs, arc->parent_pt, arc->segi, narcr, arc->new_sa + arc->new_da, -arc->new_da);
 				else
-					narc = grbs_arc_new(grbs, arc->parent_pt, arc->segi, narcr, arc->new_sa, arc->new_da, arc->concave);
+					narc = grbs_arc_new(grbs, arc->parent_pt, arc->segi, narcr, arc->new_sa, arc->new_da);
 
 				arc->new_r = arc->new_sa = arc->new_da = 0;
 				arc->new_in_use = 0;
@@ -511,18 +430,9 @@ static grbs_arc_t *grbs_path_realize_(grbs_t *grbs, grbs_2net_t *tn, grbs_addr_t
 			if ((arc == first) && !dry)
 				update_seg_sentinel_angles(first, narc);
 
-			/* if we inserted a convex, we need to bump any concave above it as well */
-			if (!is_concave) {
-				/* have to set these up temporarily, because first concave radius
-				   calculation depends on these if our arc is the highest orbit convex */
-				narc->copper = tn->copper;
-				narc->clearance = tn->clearance;
+			narc->copper = tn->copper;
+			narc->clearance = tn->clearance;
 
-				bump_seg_radii_concave_above_convex(grbs, first, dry, &coll_tmp);
-				coll |= coll_tmp;
-			}
-
-#warning TODO: line coll on the last line
 			if (!dry) {
 				grbs_arc_bbox(narc);
 				if (coll_check_arc(grbs, tn, narc, 0)) {
@@ -537,11 +447,10 @@ static grbs_arc_t *grbs_path_realize_(grbs_t *grbs, grbs_2net_t *tn, grbs_addr_t
 			pt = addr->obj.pt;
 			/* this will insert the new arc in the ->incs list becuase r=0 (segi doesn't matter) */
 			if (!dry)
-				narc = grbs_arc_new(grbs, pt, 0, 0, 0, 0, 0);
+				narc = grbs_arc_new(grbs, pt, 0, 0, 0, 0);
 			else
 				narc = &narc_dry;
 
-#warning TODO: line coll
 			break;
 
 
@@ -591,9 +500,6 @@ static grbs_arc_t *grbs_path_realize_(grbs_t *grbs, grbs_2net_t *tn, grbs_addr_t
 			CHG_NEW(grbs, narc);
 		grbs_line_create(grbs, narc);
 	}
-	else {
-#warning TODO: dry: check line
-	}
 
 	/* when the second arc is placed, go back and calc the incident angle of
 	   the first which was unknown by the time it was created, if route start
@@ -627,8 +533,8 @@ grbs_line_t *grbs_line_realize(grbs_t *grbs, grbs_2net_t *tn, grbs_point_t *p1,
 	grbs_arc_t *a1, *a2;
 	grbs_line_t *l;
 
-	a1 = grbs_arc_new(grbs, p1, 0, 0, 0, 0, 0);
-	a2 = grbs_arc_new(grbs, p2, 0, 0, 0, 0, 0);
+	a1 = grbs_arc_new(grbs, p1, 0, 0, 0, 0);
+	a2 = grbs_arc_new(grbs, p2, 0, 0, 0, 0);
 
 	gdl_append(&tn->arcs, a1, link_2net);
 	gdl_append(&tn->arcs, a2, link_2net);
@@ -656,7 +562,7 @@ int grbs_path_validate(grbs_t *grbs, grbs_2net_t *tn, grbs_addr_t *prev_addr, gr
 		case ADDR_ARC_CONVEX:
 
 		/* check for self intersecting lines */
-		if ((arc->new_da <= M_PI) && (arc->new_da >= -M_PI))
+		if ((arc->new_da <= GRBS_PI) && (arc->new_da >= -GRBS_PI))
 			return 0; /* no intersection possible if angle is below 180 deg */
 
 		/* do not do anything if there are no two exit lines */
@@ -710,7 +616,7 @@ void grbs_path_cleanup_all(grbs_t *grbs)
 		int tune = (a->new_in_use && (a == first));
 		next = gdl_next(&grbs->all_arcs, a);
 		a->new_in_use = 0;
-		grbs_clean_unused_sentinel_seg(grbs, a->parent_pt, a->concave, a->segi, tune);
+		grbs_clean_unused_sentinel_seg(grbs, a->parent_pt, a->segi, tune);
 	}
 	uall_stacks_clean(&grbs->stk_addrs);
 }
@@ -727,9 +633,8 @@ static void grbs_path_cleanup_addr_(grbs_t *grbs, grbs_addr_t *addr, int call_cn
 			grbs_path_cleanup_addr_(grbs, addr->last_real, 0);
 			break;
 		case ADDR_ARC_CONVEX:
-		case ADDR_ARC_CONCAVE:
 			addr->obj.arc->new_in_use = 0;
-			grbs_clean_unused_sentinel_seg(grbs, addr->obj.arc->parent_pt, addr->obj.arc->concave, addr->obj.arc->segi, (addr->obj.arc->link_point.prev == NULL));
+			grbs_clean_unused_sentinel_seg(grbs, addr->obj.arc->parent_pt, addr->obj.arc->segi, (addr->obj.arc->link_point.prev == NULL));
 			if ((call_cnt < 2) && (addr->last_real != NULL))
 				grbs_path_cleanup_addr_(grbs, addr->last_real, call_cnt+1);
 			break;
