@@ -1,3 +1,26 @@
+/*  libgrbs - geometric rubber band sketch model
+    Copyright (C) 2021  Tibor 'Igor2' Palinkas
+    (Supported by NLnet NGI0 PET Fund in 2021)
+
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+    Contact:
+      Project page: http://repo.hu/projects/libgrbs
+      lead developer: http://repo.hu/projects/pcb-rnd/contact.html
+*/
+
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -336,7 +359,7 @@ static int path_parse_append(grbs_2net_t *tn, char *args)
 
 	grbs_addr_t *addr;
 	char *s, *cmd, *pt, *next;
-	int curr_dir, res = -1, concave;
+	int curr_dir, res = -1;
 	grbs_point_t *curr = NULL;
 
 
@@ -362,15 +385,8 @@ static int path_parse_append(grbs_2net_t *tn, char *args)
 		}
 
 
-		concave = 0;
 		curr_dir = -1;
 
-		/* decode optional cmd prefix */
-		if (strcmp(cmd, "concave") == 0) {
-			concave = 10;
-			shift_word(s, next); cmd = s; s = next; /* affects the next word */
-		}
-
 		shift_word(s, next); pt = s;
 
 		if (*cmd == '\0')
@@ -390,9 +406,9 @@ static int path_parse_append(grbs_2net_t *tn, char *args)
 		else if ((strcmp(cmd, "from") == 0) || (strcmp(cmd, "to") == 0))
 			curr_dir = GRBS_ADIR_INC;
 		else if (strcmp(cmd, "cw") == 0)
-			curr_dir = GRBS_ADIR_CONVEX_CW + concave;
+			curr_dir = GRBS_ADIR_CONVEX_CW;
 		else if (strcmp(cmd, "ccw") == 0)
-			curr_dir = GRBS_ADIR_CONVEX_CCW + concave;
+			curr_dir = GRBS_ADIR_CONVEX_CCW;
 		else if (strcmp(cmd, "vconcave") == 0) {
 			curr_dir = GRBS_ADIR_VCONCAVE;
 		}
@@ -531,6 +547,26 @@ static int cmd_2net_markpt(char *args)
 	return 0;
 }
 
+static int cmd_imline(char *args)
+{
+	char *name = args, *pt1name, *pt2name;
+	grbs_point_t *pt1, *pt2;
+	grbs_2net_t *tn;
+	grbs_line_t *l;
+
+	pt1name = args;
+	shift_name(args, 0);
+	obj_by_name(pt1, pt1name, "no point1");
+
+	pt2name = name = args;
+	shift_name(args, 0);
+	obj_by_name(pt2, pt2name, "no point2");
+
+	tn = grbs_2net_new(&ctx, 0, 0);
+	l = grbs_line_realize(&ctx, tn, pt1, pt2);
+	l->immutable = 1;
+	return 0;
+}
 
 static int cmd_geo_bicycle(char *args)
 {
@@ -607,12 +643,6 @@ static int cmd_brk(char *args)
 }
 
 
-static int cmd_pp_concave(char *cmd)
-{
-	grbs_pp_concave_corners(&ctx);
-	return 0;
-}
-
 static int parse_cmd(char *cmd, char *args)
 {
 	if (*cmd == '\0')                     return 0;
@@ -634,11 +664,11 @@ static int parse_cmd(char *cmd, char *args)
 	if (strcmp(cmd, "2net_append") == 0)  return cmd_2net_append(args);
 	if (strcmp(cmd, "2net_cleanup") == 0) return cmd_2net_cleanup(args);
 	if (strcmp(cmd, "2net_markpt") == 0)  return cmd_2net_markpt(args);
+	if (strcmp(cmd, "imline") == 0)       return cmd_imline(args);
 	if (strcmp(cmd, "geo_bicycle") == 0)  return cmd_geo_bicycle(args);
 	if (strcmp(cmd, "echo") == 0)         return cmd_echo(args);
 	if (strcmp(cmd, "fail") == 0)         return cmd_fail(args);
 	if (strcmp(cmd, "brk") == 0)          return cmd_brk(args);
-	if (strcmp(cmd, "pp_concave") == 0)   return cmd_pp_concave(args);
 	if (strcmp(cmd, "exit") == 0)         return ERR_EXIT;
 
 
